@@ -1,7 +1,7 @@
 package Dist::Zilla::Plugin::Rinci::Validate;
 
-our $DATE = '2016-02-25'; # DATE
-our $VERSION = '0.21'; # VERSION
+our $DATE = '2016-06-02'; # DATE
+our $VERSION = '0.22'; # VERSION
 
 use 5.010001;
 use strict;
@@ -81,7 +81,7 @@ sub munge_file {
     my %vargs; # list of validated args for current sub, val 2=skipped
     my %vsubs; # list of subs
     my %vars;    # list of variables that the generated validator needs
-    my @modules; # list of modules that the generated validator needs
+    my @modules; # list of modules (records) that the generated validator needs
 
     my $i = 0; # line number
 
@@ -134,10 +134,12 @@ sub munge_file {
             return_type => 'str',
             comment     => 0,
         );
+        die "Incompatible Data::Sah version (cd v=$cd->{v}, expected 2)" unless $cd->{v} == 2;
         my @code;
-        for (@{$cd->{modules}}) {
-            push @code, $plc->stmt_require_module($_, $cd) unless $_ ~~ @modules;
-            push @modules, $_;
+        for my $mod_rec (@{$cd->{modules}}) {
+            push @code, $plc->stmt_require_module($mod_rec) unless
+                grep { $_->{name} eq $mod_rec->{name} && !$mod_rec->{use_statement} } @modules;
+            push @modules, $mod_rec;
         }
         for (sort keys %{$cd->{vars}}) {
             push @code, "my \$$_ = ".$plc->literal($cd->{vars}{$_})."; "
@@ -177,9 +179,11 @@ sub munge_file {
                     return_type => 'str',
                     comment     => 0,
                 );
-                for (@{$cd->{modules}}) {
-                    push @code, $plc->stmt_require_module($_, $cd) unless $_ ~~ @modules;
-                    push @modules, $_;
+                die "Incompatible Data::Sah version (cd v=$cd->{v}, expected 2)" unless $cd->{v} == 2;
+                for my $mod_rec (@{$cd->{modules}}) {
+                    push @code, $plc->stmt_require_module($mod_rec) unless
+                        grep { $_->{name} eq $mod_rec->{name} && !$mod_rec->{use_statement} } @modules;
+                    push @modules, $mod_rec;
                 }
                 for (sort keys %{$cd->{vars}}) {
                     push @code, "my \$$_ = ".$plc->literal($cd->{vars}{$_})."; "
@@ -342,7 +346,7 @@ Dist::Zilla::Plugin::Rinci::Validate - Insert argument validator code in output
 
 =head1 VERSION
 
-This document describes version 0.21 of Dist::Zilla::Plugin::Rinci::Validate (from Perl distribution Dist-Zilla-Plugin-Rinci-Validate), released on 2016-02-25.
+This document describes version 0.22 of Dist::Zilla::Plugin::Rinci::Validate (from Perl distribution Dist-Zilla-Plugin-Rinci-Validate), released on 2016-06-02.
 
 =head1 SYNOPSIS
 
@@ -482,7 +486,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Dist-Zilla
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Dist-Zilla-Plugin-Rinci-Validate>.
+Source repository is at L<https://github.com/sharyanto/perl-Dist-Zilla-Plugin-Rinci-Validate>.
 
 =head1 BUGS
 
@@ -496,6 +500,8 @@ feature.
 
 L<Dist::Zilla::Plugin::Rinci::Wrap>
 
+L<Data::Sah::Manual::ParamsValidating>
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
